function yy = stateportrait(model_name,x0)
%STATEPORTRAIT State portrait of a nonlinear system.
%
% STATEPORTRAIT(MODEL_NAME) Produce a state portrait for the Simulink model named MODEL_NAME 
% using a quiver plot. The model must have two states with OUTPUT block attached.
%
% STATEPORTRAIT(MODEL_NAME,X0) Produce a state portrait for the Simulink model named MODEL_NAME 
% using a quiver plot. Additionally, a state trajectory for an initial state vector X0 is plotted.
% The model must have two states with OUTPUT block attached. 
%
% The function is based on J.B.Dabney, T.L.Harman: Mastering Simulink 2. Prentice-Hall, Inc., 1998, pp.194.

% Author(s):    Z. Hurak   11-8-2001
% $Revision: 1.0.0 $  $Date: 06-22-2001 $

%-----------------------------------------------------------------------------

%====================================================================
%Tests for proper inputs:
%====================================================================

if nargin > 2
   error('Two many input arguments.')
end

if nargin == 0
    error('Not enough input arguments.')
end

if ~isa(model_name,'char')
    error('The first input argument must be a name of a Simulink model.')
end

if (nargin == 2) & ((~isa(x0,'double')) | (length(x0)~=2))
    error('Second input argument must be a vector of initial conditions X0 = [x01 x02].')
end

%====================================================================
%Quiver plot:
%====================================================================
h = 0.01;
opts = simset('Solver','ode5','FixedStep',h);
x1 = -2.5:0.25:2.5;
x2 = -2.5:0.25:2.5;
[nr,nc] = size(x1);
x1m = zeros(nc,nc);
x2m = x1m;
for nx1 = 1:nc
   for nx2 = 1:nc
      opts = simset(opts,'InitialState',[x1(nx1),x2(nx2)]);
      [t,x,y] = sim(model_name,h,opts);
      dx1 = x(2,1)-x1(nx1);
      dx2 = x(2,2)-x2(nx2);
      l = sqrt(dx1^2 + dx2^2)*7.5;
      if l > 1.0e-10
         x1m(nx2,nx1)=dx1/l;
         x2m(nx2,nx1)=dx2/l;
      end
   end
end
quiver(x1,x2,x1m,x2m,0);
axis([-2.5, 2.5, -2.5, 2.5]);
xlabel('x1');
ylabel('x2');
grid;
tit = ['State portrait of ',model_name,' system'];
title(tit);
axis manual

%====================================================================
%Plot the optional state trajectory, if required:
%====================================================================

if ~isempty(x0)
    hold on;
    opts = simset('InitialState',x0);
    [t,x,y] = sim(model_name,15,opts);
    plot(x(:,1),x(:,2),'r');
    h_StateTrajectory = findobj(gcf,'Color','red');
    set(h_StateTrajectory,'LineWidth',3);
    hold off
end
